import Avatar from '@mui/material/Avatar'
import Button from '@mui/material/Button'
import TextField from '@mui/material/TextField'
import Link from '@mui/material/Link'
import Grid from '@mui/material/Unstable_Grid2'
import AppRegistrationIcon from '@mui/icons-material/AppRegistration'
import Typography from '@mui/material/Typography'
import Container from '@mui/material/Container'
import { useRouter } from 'next/router'
import * as yup from '@/utils/validation'
import * as api from '@/api'
import { RegisterInputDto } from '@/api/user.interface'
import { EmailVerifyCodeScene } from '@/api/email.interface'
import { useCountdown, COUNTDOWN_SECONDS } from '@/utils/useCountdown'
import { useForm, SubmitHandler } from 'react-hook-form'
import { yupResolver } from '@hookform/resolvers/yup'

export default function Register() {
  const router = useRouter()
  const { countdown, setCountdown } = useCountdown()

  const hookForm = useForm<RegisterInputDto>({
    mode: 'onTouched',
    resolver: yupResolver(
      yup.object({
        email: yup.emailSchema,
        password: yup.passwordSchema,
        verifyCode: yup.verifyCodeSchema,
      }),
    ),
  })
  const onSubmit: SubmitHandler<RegisterInputDto> = async (values) => {
    const res = await api.user.register(values)
    localStorage.setItem('accessToken', res.data)
    router.push('/')
  }

  async function sendVerifyCode() {
    const isValid = await hookForm.trigger('email')
    if (!isValid) return
    setCountdown(COUNTDOWN_SECONDS)
    try {
      const res = await api.email.sendEmailVerifyCode({
        email: hookForm.getValues('email'),
        scene: EmailVerifyCodeScene.register,
      })
      hookForm.setValue('token', res.data.token)
    } catch (err) {
      setCountdown(0)
    }
  }

  return (
    <Container
      component="main"
      maxWidth="xs"
      sx={{
        marginTop: '20vh',
        display: 'flex',
        flexDirection: 'column',
        alignItems: 'center',
      }}
    >
      <Avatar sx={{ bgcolor: 'primary.main' }}>
        <AppRegistrationIcon />
      </Avatar>
      <Typography component="h1" variant="h5" mt={1} mb={3}>
        注册
      </Typography>
      <Grid
        container
        component="form"
        spacing={[2, 1]}
        onSubmit={hookForm.handleSubmit(onSubmit)}
      >
        <Grid xs={12}>
          <TextField
            {...hookForm.register('email')}
            required
            error={!!hookForm.formState.errors.email}
            helperText={hookForm.formState.errors.email?.message || ' '}
            fullWidth
            label="邮箱"
            autoComplete="email"
            autoFocus
            sx={{ mt: 1 }}
          />
        </Grid>
        <Grid xs={8}>
          <TextField
            {...hookForm.register('verifyCode')}
            required
            error={!!hookForm.formState.errors.verifyCode}
            helperText={hookForm.formState.errors.verifyCode?.message || ' '}
            fullWidth
            label="验证码"
          />
        </Grid>
        <Grid xs={4} mt={1}>
          <Button
            size="large"
            fullWidth
            variant="contained"
            disabled={countdown > 0}
            onClick={sendVerifyCode}
          >
            {countdown > 0 ? countdown : '发送验证码'}
          </Button>
        </Grid>
        <Grid xs={12}>
          <TextField
            {...hookForm.register('password')}
            required
            error={!!hookForm.formState.errors.password}
            helperText={hookForm.formState.errors.password?.message || ' '}
            fullWidth
            label="密码"
            type="password"
            autoComplete="current-password"
          />
        </Grid>
        <Grid xs={12}>
          <Button type="submit" fullWidth variant="contained">
            注册
          </Button>
        </Grid>
        <Grid xs={12} container justifyContent="flex-end" sx={{ mt: 2 }}>
          <Grid>
            <Typography component="span" variant="body2">
              已有账号？
            </Typography>
            <Link href="/login">登录</Link>
          </Grid>
        </Grid>
      </Grid>
    </Container>
  )
}
