import axios, { AxiosError, type AxiosResponse } from 'axios'
import Router from 'next/router'
import status from 'http-status'
import { AccessToken } from '@/api/user.interface'
import { throwError, ErrorResponse } from './throwError'
import normalAxios from './normalAxios'

/** 需要Authorization的axios */
const authAxios = axios.create()

authAxios.interceptors.request.use(function (config) {
  const accessToken = localStorage.getItem('accessToken')
  config.headers.Authorization = `Bearer ${accessToken}`
  return config
})

authAxios.interceptors.response.use(
  (response) => response,
  async (error: AxiosError<ErrorResponse>) => {
    switch (error.response?.status) {
      case status.UNAUTHORIZED: {
        const res = await refreshToken()
        localStorage.setItem('accessToken', res.data)
        return error.config && axios.request(error.config)
      }
      default: {
        return throwError(error)
      }
    }
  },
)

const refreshToken = (() => {
  let refreshingToken: Promise<AxiosResponse<AccessToken>> | null = null
  return async function () {
    if (!refreshingToken) {
      refreshingToken = normalAxios
        .put<AccessToken>('/api/users/me/token')
        .finally(() => {
          refreshingToken = null
        })
    }
    refreshingToken?.catch((error) => {
      if (error instanceof AxiosError) {
        Router.push('/login')
        console.error(error.response?.data.message)
      }
    })
    return refreshingToken
  }
})()

export default authAxios
